/*
 * Copyright (C) 2002-2003 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin;

import java.util.*;

import org.jdom.Element;

import com.intellij.openapi.project.Project;
import com.kiwisoft.db.DatabaseManager;
import com.kiwisoft.db.QueryManager;
import com.kiwisoft.utils.NotifyObject;
import com.kiwisoft.utils.Utils;

/**
 * @author Sven Krause
 * @author Stefan Stiller
 * @version $Revision: 1.2 $, $Date: 2004/11/21 11:21:20 $
 */
public class CommonConfiguration extends Observable
{
	public static final String GENERAL="general";
	public static final String CONFIRM_COMMIT="confirmCommit";
	public static final String CONFIRM_ROLLBACK="confirmRollback";
	public static final String CONFIRM_DISCONNECT="confirmClose";

	public static final String DATABASES="databases";

	public static final String SAVE_PASSWORDS="savePasswords";
	public static final String ENCODE_PASSWORDS="encodePasswords";

	public static final String RESULTS="results";
	public static final String RESIZE_COLUMNS="resizeColumns";
	public static final String TO_HEADER="toHeader";
	public static final String TO_CONTENT="toContent";
	public static final String NULL_STRING="nullString";
	public static final String PRIMARY_KEY_COLUMNS="primaryKeyCols";
	public static final String ROW_LIMIT="rowLimit";
	public static final String DEFAULT_FORMAT="defaultFormat";
	public static final String CLASS="class";
	public static final String FORMAT="format";
	public static final String SAVE_TABLE_CONFIG="saveTableConfig";

	public static final String QUERIES="queries";
	public static final String SAVE="save";
	public static final String PATH="path";
	public static final String PARTIAL_EXECUTE="partialExecute";
	public static final String PROJECT_CLASS_LOADER="projectClassLoader";

	public static final String VALUE="value";
	public static final String ENABLED="enabled";

	private static Map instanceMap=new WeakHashMap();

	public static CommonConfiguration getInstance(Project project)
	{
		CommonConfiguration instance=(CommonConfiguration)instanceMap.get(project);
		if (instance==null)
		{
			instance=new CommonConfiguration();
			instanceMap.put(project, instance);
		}
		return instance;
	}

	public static void closeInstance(Project project)
	{
		instanceMap.remove(project);
	}

	private String queryPath;
	private boolean saveQueries;

	private boolean rowLimitEnabled=true;
	private int rowLimit=1000;
	private boolean showPrimaryKeyColumns=true;
	private boolean savePasswords;
	private boolean encodePasswords=true;
	private boolean resizeColumnsToHeader=true;
	private boolean resizeColumnsToContent=true;
	private boolean saveResultTableConfiguration;
	private boolean includeProjectClasses;
	private String nullString;
	private Map confirmations=new HashMap();
	private Map defaultFormats=new HashMap();
	private boolean loadLOBs;

	public boolean isIncludeProjectClasses()
	{
		return includeProjectClasses;
	}

	public void setIncludeProjectClasses(boolean includeProjectClasses)
	{
		this.includeProjectClasses=includeProjectClasses;
	}

	public String getNullString()
	{
		return nullString;
	}

	public void setNullString(String nullString)
	{
		this.nullString=nullString;
		setChanged();
		notifyObservers(new NotifyObject("nullString changed"));
	}

	public Boolean getConfirmation(String key)
	{
		return (Boolean)confirmations.get(key);
	}

	public void setConfirmation(String key, Boolean value)
	{
		confirmations.put(key, value);
	}

	public int getRowLimit()
	{
		return rowLimit;
	}

	public void setRowLimit(int rowLimit)
	{
		if (rowLimit<0) this.rowLimit=1000;
		this.rowLimit=rowLimit;
	}

	public void setRowLimitEnabled(boolean rowLimitEnabled)
	{
		this.rowLimitEnabled=rowLimitEnabled;
	}

	public boolean isRowLimitEnabled()
	{
		return rowLimitEnabled;
	}

	public boolean isSaveResultTableConfiguration()
	{
		return saveResultTableConfiguration;
	}

	public void setSaveResultTableConfiguration(boolean saveResultTableConfiguration)
	{
		this.saveResultTableConfiguration=saveResultTableConfiguration;
	}

	public void setDefaultFormat(Class aClass, String format)
	{
		String oldFormat=getDefaultFormat(aClass);
		defaultFormats.put(aClass.getName(), format);
		if (oldFormat!=null ? !oldFormat.equals(format) : format!=null)
		{
			setChanged();
			notifyObservers(new NotifyObject("defaultFormat changed", aClass, format));
		}
	}

	public String getDefaultFormat(Class aClass)
	{
		return (String)defaultFormats.get(aClass.getName());
	}

	public boolean isSaveQueries()
	{
		return saveQueries;
	}

	public void setSaveQueries(boolean saveQueries)
	{
		this.saveQueries=saveQueries;
	}

	public boolean isShowPrimaryKeyColumns()
	{
		return showPrimaryKeyColumns;
	}

	public void setShowPrimaryKeyColumns(boolean showPrimaryKeyColumns)
	{
		this.showPrimaryKeyColumns=showPrimaryKeyColumns;
	}

	public boolean isSavePasswords()
	{
		return savePasswords;
	}

	public void setSavePasswords(boolean savePasswords)
	{
		this.savePasswords=savePasswords;
	}

	public boolean isEncodePasswords()
	{
		return encodePasswords;
	}

	public void setEncodePasswords(boolean encodePasswords)
	{
		this.encodePasswords=encodePasswords;
	}

	public void setDriversPath(String driversPath)
	{
	}

	public String getQueryPath()
	{
		return queryPath;
	}

	public void setQueryPath(String queryPath)
	{
		this.queryPath=queryPath;
	}

	public boolean isResizeColumnsToHeader()
	{
		return resizeColumnsToHeader;
	}

	public void setResizeColumnsToHeader(boolean resizeColumnsToHeader)
	{
		this.resizeColumnsToHeader=resizeColumnsToHeader;
	}

	public boolean isResizeColumnsToContent()
	{
		return resizeColumnsToContent;
	}

	public void setResizeColumnsToContent(boolean resizeColumnsToContent)
	{
		this.resizeColumnsToContent=resizeColumnsToContent;
	}

	public void readExternal(Element element, Project project)
	{
		Element generalElement=element.getChild(GENERAL);
		if (generalElement!=null)
		{
			setConfirmation(CONFIRM_COMMIT, Utils.getBoolean(generalElement, CONFIRM_COMMIT, null));
			setConfirmation(CONFIRM_ROLLBACK, Utils.getBoolean(generalElement, CONFIRM_ROLLBACK, null));
			setConfirmation(CONFIRM_DISCONNECT, Utils.getBoolean(generalElement, CONFIRM_DISCONNECT, null));
		}

		Element databasesElement=element.getChild(DATABASES);
		if (databasesElement!=null)
		{
			setSavePasswords(Utils.getBoolean(databasesElement, SAVE_PASSWORDS, false));
			setEncodePasswords(Utils.getBoolean(databasesElement, ENCODE_PASSWORDS, true));
			DatabaseManager.getInstance(project).readExternal(databasesElement, project);
		}
		else
			DatabaseManager.getInstance(project).readExternal(element, project);

		Element queriesElement=element.getChild(QUERIES);
		if (queriesElement!=null)
		{
			setConfirmation(PARTIAL_EXECUTE, Utils.getBoolean(queriesElement, PARTIAL_EXECUTE, null));
			setQueryPath(queriesElement.getAttributeValue(PATH));
			setSaveQueries(Utils.getBoolean(queriesElement, SAVE, false));
			setIncludeProjectClasses(Utils.getBoolean(queriesElement, PROJECT_CLASS_LOADER, false));
			QueryManager.getInstance(project).readExternal(queriesElement);
		}

		Element resultsElement=element.getChild(RESULTS);
		if (resultsElement!=null)
		{
			setShowPrimaryKeyColumns(Utils.getBoolean(resultsElement, PRIMARY_KEY_COLUMNS, true));
			setLoadLargeObjects(Utils.getBoolean(resultsElement, "loadLOBs", true));
			setSaveResultTableConfiguration(Utils.getBoolean(resultsElement, SAVE_TABLE_CONFIG, false));
			List children=resultsElement.getChildren(DEFAULT_FORMAT);
			if (children!=null)
			{
				for (Iterator it=children.iterator(); it.hasNext();)
				{
					Element formatElement=(Element)it.next();
					String className=formatElement.getAttributeValue(CLASS);
					String format=formatElement.getAttributeValue(FORMAT);
					defaultFormats.put(className, format);
				}
			}
			setNullString(resultsElement.getAttributeValue(NULL_STRING));

			Element rowLimitElement=resultsElement.getChild(ROW_LIMIT);
			setRowLimitEnabled(Utils.getBoolean(rowLimitElement, ENABLED, true));
			setRowLimit(Utils.getInteger(rowLimitElement, VALUE, new Integer(1000)).intValue());

			Element resizeElement=element.getChild(RESIZE_COLUMNS);
			if (resizeElement!=null)
			{
				setResizeColumnsToHeader(Utils.getBoolean(resizeElement, TO_HEADER, true));
				setResizeColumnsToContent(Utils.getBoolean(resizeElement, TO_CONTENT, true));
			}
		}
	}

	public void writeExternal(Element element, Project project)
	{
		// General options
		Element generalChild=new Element(GENERAL);
		Utils.setValue(generalChild, CONFIRM_COMMIT, getConfirmation(CONFIRM_COMMIT));
		Utils.setValue(generalChild, CONFIRM_ROLLBACK, getConfirmation(CONFIRM_ROLLBACK));
		Utils.setValue(generalChild, CONFIRM_DISCONNECT, getConfirmation(CONFIRM_DISCONNECT));
		element.addContent(generalChild);

		// Databases
		Element databasesChild=new Element(DATABASES);
		databasesChild.setAttribute(SAVE_PASSWORDS, Boolean.toString(isSavePasswords()));
		databasesChild.setAttribute(ENCODE_PASSWORDS, Boolean.toString(isEncodePasswords()));
		DatabaseManager.getInstance(project).writeExternal(databasesChild, project);
		element.addContent(databasesChild);

		// Queries
		Element child=new Element(QUERIES);
		Utils.setValue(child, PATH, getQueryPath());
		Utils.setValue(child, PARTIAL_EXECUTE, getConfirmation(PARTIAL_EXECUTE));
		child.setAttribute(PROJECT_CLASS_LOADER, Boolean.toString(isIncludeProjectClasses()));
		child.setAttribute(SAVE, Boolean.toString(isSaveQueries()));
		if (isSaveQueries()) QueryManager.getInstance(project).writeExternal(child);
		element.addContent(child);

		// Results
		Element resultsChild=new Element(RESULTS);
		resultsChild.setAttribute(PRIMARY_KEY_COLUMNS, Boolean.toString(isShowPrimaryKeyColumns()));
		resultsChild.setAttribute("loadLOBs", Boolean.toString(isShowPrimaryKeyColumns()));
		resultsChild.setAttribute(SAVE_TABLE_CONFIG, Boolean.toString(isSaveResultTableConfiguration()));
		for (Iterator it=defaultFormats.keySet().iterator(); it.hasNext();)
		{
			String className=(String)it.next();
			Element formatElement=new Element(DEFAULT_FORMAT);
			Utils.setValue(formatElement, CLASS, className);
			Utils.setValue(formatElement, FORMAT, defaultFormats.get(className));
			resultsChild.addContent(formatElement);
		}
		Utils.setValue(resultsChild, NULL_STRING, getNullString());
		element.addContent(resultsChild);

		child=new Element(ROW_LIMIT);
		child.setAttribute(ENABLED, Boolean.toString(isRowLimitEnabled()));
		child.setAttribute(VALUE, Integer.toString(getRowLimit()));
		resultsChild.addContent(child);

		child=new Element(RESIZE_COLUMNS);
		child.setAttribute(TO_HEADER, Boolean.toString(isResizeColumnsToHeader()));
		child.setAttribute(TO_CONTENT, Boolean.toString(isResizeColumnsToContent()));
		resultsChild.addContent(child);
	}

	public boolean isLoadLargeObjects()
	{
		return loadLOBs;
	}

	public void setLoadLargeObjects(boolean newValue)
	{
		loadLOBs=newValue;
	}

}
