/*
 * Copyright (C) 2002-2003 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin;

import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.*;
import java.util.*;
import java.util.List;
import javax.swing.*;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import com.intellij.openapi.project.Project;

import com.kiwisoft.db.BLOBWrapper;
import com.kiwisoft.db.CLOBWrapper;
import com.kiwisoft.db.ColumnData;
import com.kiwisoft.db.QueryManager;
import com.kiwisoft.db.sql.SQLStatement;
import com.kiwisoft.db.sql.SystemStatement;
import com.kiwisoft.utils.SetMap;
import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.Utils;
import com.kiwisoft.utils.format.ObjectFormat;
import com.kiwisoft.utils.gui.*;
import com.kiwisoft.utils.gui.inspect.InspectorDialogStub;
import com.kiwisoft.utils.gui.table.DynamicTable;
import com.kiwisoft.utils.gui.table.RendererFactory;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.11 $, $Date: 2005/04/06 20:05:52 $
 */
public class ResultSetPanel extends JPanel
{
	private DynamicTable tblResult;
	private QueryPanel queryPanel;
	private Project project;
	private AutoRefreshThread autoRefreshThread;

	public ResultSetPanel(Project project, QueryPanel queryPanel, ResultSetTableModel tableModel, String statusText)
	{
		this.project=project;
		this.queryPanel=queryPanel;
		CommonConfiguration configuration=CommonConfiguration.getInstance(project);

		JLabel lblResult=new JLabel(statusText);

		tblResult=new DynamicTable(tableModel);
		tblResult.putClientProperty(DynamicTable.NULL_STRING, configuration.getNullString());
		tblResult.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		tblResult.setCellSelectionEnabled(true);
		ResultSelectionListener resultSelectionListener=new ResultSelectionListener();
		tblResult.addMouseListener(resultSelectionListener);
		tblResult.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "select");
		tblResult.getActionMap().put("select", resultSelectionListener);
		tblResult.getTableHeader().addMouseListener(new TableHeaderMouseListener());
		ResultTableConfiguration tableConfiguration=tableModel.getTableConfiguration();
		tblResult.initializeColumns(tableConfiguration);
		if (!tableConfiguration.isSupportWidths())
			tblResult.sizeColumnsToFit(configuration.isResizeColumnsToHeader(), configuration.isResizeColumnsToContent());
		tableConfiguration.setInitalized(true);

		setLayout(new BorderLayout());
		add(new JScrollPane(tblResult), BorderLayout.CENTER);
		add(lblResult, BorderLayout.SOUTH);
	}

	public void removeNotify()
	{
		if (autoRefreshThread!=null && autoRefreshThread.isAlive()) autoRefreshThread.setStopped(true);
		super.removeNotify();
	}

	private class ResultSelectionListener extends AbstractAction implements MouseListener
	{
		public void actionPerformed(ActionEvent e)
		{
			showSelection();
		}

		public void mouseClicked(MouseEvent e)
		{
			if (e.getClickCount()>1 && e.getButton()==MouseEvent.BUTTON1)
			{
				showSelection();
				e.consume();
			}
			else if (e.getSource()==tblResult && (e.isPopupTrigger() || e.getButton()==MouseEvent.BUTTON3))
			{
				int selectedRow=tblResult.rowAtPoint(e.getPoint());
				int selectedColumn=tblResult.columnAtPoint(e.getPoint());
				selectedColumn=tblResult.convertColumnIndexToModel(selectedColumn);
				ResultSetTableModel tableModel=null;
				if (tblResult.getModel() instanceof ResultSetTableModel) tableModel=(ResultSetTableModel)tblResult.getModel();

				JPopupMenu popup=new JPopupMenu("Result");
				popup.add(new CopyToClipboardAction(tblResult.getSelectedRows(), tblResult.getSelectedColumns()));
				popup.add(new CellStatisticsAction(tblResult.getSelectedRows(), tblResult.getSelectedColumns()));
				popup.add(new ShowReferencedRowAction(selectedRow, selectedColumn));
				if (selectedRow>=0 && selectedColumn>=0 && tableModel!=null)
				{
					ColumnInfo info=tableModel.getColumnInfo(selectedColumn);
					JMenu menu=new JMenu("Show referencing rows");
					menu.setIcon(IconManager.ICON_16X16);
					popup.add(menu);
					if (info.isExportedKey())
					{
						Object value=tableModel.getValueAt(selectedRow, selectedColumn);
						for (Iterator it=info.getExportedKeys().iterator(); it.hasNext();)
						{
							ColumnData columnData=(ColumnData)it.next();
            				menu.add(new ShowReferencingRowsAction(info, value, columnData));
						}
					}
					if (menu.getMenuComponentCount()==0) menu.add(new EmptyAction());
				}
				popup.addSeparator();
				popup.add(new RefreshAction());
				popup.add(new AutoRefreshAction());

				if (selectedRow>=0 && selectedColumn>=0 && tableModel!=null)
				{
					Class type=tableModel.getCellClass(selectedRow, selectedColumn);
					String currentVariant=tableModel.getCellFormat(selectedRow, selectedColumn);
					if (currentVariant==null) currentVariant=ObjectFormat.DEFAULT;
					SetMap variants=RendererFactory.getInstance().getVariants(type);

					if (!variants.isEmpty())
					{
						JMenu formatMenu=new JMenu("Format");
						formatMenu.setIcon(IconManager.ICON_16X16);
						JRadioButtonMenuItem menuItem=new JRadioButtonMenuItem(new SetFormatAction(selectedColumn, ObjectFormat.DEFAULT));
						menuItem.setSelected(StringUtils.equal(currentVariant, ObjectFormat.DEFAULT));
						formatMenu.add(menuItem);
						variants.remove(null, ObjectFormat.DEFAULT);

						if (!variants.isEmpty()) formatMenu.addSeparator();

						for (Iterator it=new TreeSet(variants.remove(null)).iterator(); it.hasNext();)
						{
							String variant=(String)it.next();
							menuItem=new JRadioButtonMenuItem(new SetFormatAction(selectedColumn, variant));
							menuItem.setSelected(StringUtils.equal(currentVariant, variant));
							formatMenu.add(menuItem);
						}
						for (Iterator itGroup=new TreeSet(variants.keySet()).iterator(); itGroup.hasNext();)
						{
							String group=(String)itGroup.next();
							JMenu groupMenu=new JMenu(group);
							formatMenu.add(groupMenu);
							for (Iterator it=new TreeSet(variants.get(group)).iterator(); it.hasNext();)
							{
								String variant=(String)it.next();
								menuItem=new JRadioButtonMenuItem(new SetFormatAction(selectedColumn, variant));
								menuItem.setSelected(StringUtils.equal(currentVariant, variant));
								groupMenu.add(menuItem);
							}
						}

						popup.addSeparator();
						popup.add(formatMenu);

						ObjectStyle cellStyle=tableModel.getCellStyle(selectedRow, selectedColumn);
						Font currentFont=cellStyle!=null ? cellStyle.getFont() : null;
						if (currentFont==null) currentFont=tblResult.getFont();
						String currentFontFamily=currentFont!=null ? currentFont.getFamily() : null;
						popup.add(new SetFontAction(selectedColumn, currentFontFamily));
					}
					popup.addSeparator();
					popup.add(new SetPrimaryKeyAction(selectedColumn));
				}

				e.consume();
				popup.show(tblResult, e.getX(), e.getY());
			}
		}

		private void showSelection()
		{
			int column=tblResult.getSelectedColumn();
			int row=tblResult.getSelectedRow();
			Object value=tblResult.getValueAt(row, column);
			if (value instanceof BLOBWrapper)
			{
				BLOBWrapper blobWrapper=(BLOBWrapper)value;
				if (blobWrapper.isLoaded())
				{
					Object object=blobWrapper.getObject();
					if (object!=null)
						Utils.wrapDialog(project, new InspectorDialogStub("Binary Large Object", object), false, true);
					else
						Utils.wrapDialog(project, new ByteArrayDialogStub("Binary Large Object", blobWrapper.getData()), false, true);
				}
			}
			else if (value instanceof CLOBWrapper)
			{
				CLOBWrapper clobWrapper=(CLOBWrapper)value;
				if (clobWrapper.isLoaded())
				{
					String text=clobWrapper.getText();
					Utils.wrapDialog(project, new TextDialogStub("Character Large Object", text), false, true);
				}
			}
			else if (value instanceof String)
			{
				Utils.wrapDialog(project, new TextDialogStub("Text", (String)value), false, true);
			}
		}

		public void mousePressed(MouseEvent e)
		{
		}

		public void mouseReleased(MouseEvent e)
		{
		}

		public void mouseEntered(MouseEvent e)
		{
		}

		public void mouseExited(MouseEvent e)
		{
		}
	}

	private class TableHeaderMouseListener extends MouseAdapter
	{
		public void mouseClicked(MouseEvent e)
		{
			if (e.getButton()==MouseEvent.BUTTON3)
			{
				JTableHeader tableHeader=(JTableHeader)e.getSource();
				int column=tableHeader.columnAtPoint(e.getPoint());
				if (column>=0)
				{
					int modelColumn=tblResult.convertColumnIndexToModel(column);
					if (tblResult.getModel() instanceof ResultSetTableModel)
					{
						JPopupMenu popup=new JPopupMenu("Result");
						popup.add(new SetPrimaryKeyAction(modelColumn));
						popup.add(new ColumnStatisticsAction(modelColumn));
						popup.addSeparator();
						String columnName=String.valueOf(tblResult.getColumnModel().getColumn(column).getHeaderValue());
						popup.add(new CopyColumnNameAction(columnName));
						popup.show(tableHeader, e.getX(), e.getY());
					}
				}
				e.consume();
			}
		}
	}

	private class SetPrimaryKeyAction extends AbstractAction
	{
		private int col;

		public SetPrimaryKeyAction(int col)
		{
			super("Mark column as primary key", IconManager.getIcon(ResultSetPanel.class.getClassLoader(), "icons/browser/primary_key.gif"));
			if (tblResult.getModel() instanceof ResultSetTableModel)
			{
				ResultSetTableModel model=(ResultSetTableModel)tblResult.getModel();
				if (model.isPrimaryKey(col)) putValue(NAME, "Unmark column as primary key");
			}
			else
				setEnabled(false);
			this.col=col;
		}

		public void actionPerformed(ActionEvent e)
		{
			if (col>=0)
			{
				if (tblResult.getModel() instanceof ResultSetTableModel)
				{
					ResultSetTableModel model=(ResultSetTableModel)tblResult.getModel();
					model.setPrimaryKey(col, !model.isPrimaryKey(col));
				}
			}
		}
	}

	private class ShowReferencedRowAction extends AbstractAction
	{
		private int col;
		private int row;

		public ShowReferencedRowAction(int row, int col)
		{
			super("Show referenced row", IconManager.ICON_16X16);
			this.col=col;
			this.row=row;
			if (row>=0 && col>=0 && tblResult.getModel() instanceof ResultSetTableModel)
			{
				ResultSetTableModel model=(ResultSetTableModel)tblResult.getModel();
				ColumnInfo columnInfo=model.getColumnInfo(col);
				ColumnData foreignKey=columnInfo.getForeignKey();
				setEnabled(foreignKey!=null);
			}
			else
				setEnabled(false);
		}

		public void actionPerformed(ActionEvent e)
		{
			if (col>=0 && tblResult.getModel() instanceof ResultSetTableModel)
			{
				ResultSetTableModel model=(ResultSetTableModel)tblResult.getModel();
				ColumnInfo columnInfo=model.getColumnInfo(col);
				ColumnData foreignKey=columnInfo.getForeignKey();
				if (foreignKey!=null)
				{
					StringBuffer table=new StringBuffer();
					if (!StringUtils.isEmpty(foreignKey.getSchema()) && !StringUtils.equal(foreignKey.getSchema(), columnInfo.getSchemaName()))
						table.append(foreignKey.getSchema()).append(".");
					table.append(foreignKey.getTable());
					Object value=model.getValueAt(row, col);
					StringBuffer sql=new StringBuffer("select * from ");
					sql.append(table);
					sql.append(" where ");
					sql.append(foreignKey.getColumn()).append("=?/*").append(value).append("*/");
					SystemStatement statement=new SystemStatement(sql.toString(), table.toString(), Collections.singletonList(value));
					QueryManager.getInstance(project).createQuery(statement);
				}
			}
		}
	}

	private class ShowReferencingRowsAction extends AbstractAction
	{
		private Object value;
		private String columnName;
		private String tableName;

		public ShowReferencingRowsAction(ColumnInfo columnInfo, Object value, ColumnData exportedColumn)
		{
			super();
			tableName="";
			if (!StringUtils.isEmpty(exportedColumn.getSchema()) && !StringUtils.equal(exportedColumn.getSchema(), columnInfo.getSchemaName()))
				tableName=tableName+exportedColumn.getSchema()+".";
			tableName=tableName+exportedColumn.getTable();
			columnName=exportedColumn.getColumn();
			putValue(NAME, tableName+"."+columnName);
			this.value=value;
		}

		public void actionPerformed(ActionEvent e)
		{
			StringBuffer sql=new StringBuffer("select * from ");
			sql.append(tableName);
			sql.append(" where ");
			sql.append(columnName).append("=?/*").append(value).append("*/");
			SystemStatement statement=new SystemStatement(sql.toString(), tableName, Collections.singletonList(value));
			QueryManager.getInstance(project).createQuery(statement);
		}
	}

	private class CopyToClipboardAction extends AbstractAction
	{
		private int[] rows;
		private int[] cols;

		public CopyToClipboardAction(int[] rows, int[] cols)
		{
			super("Copy to Clipboard", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.COPY));
			this.rows=rows;
			this.cols=cols;
			setEnabled(rows!=null && rows.length>0 && cols!=null && cols.length>0);
		}

		public void actionPerformed(ActionEvent e)
		{
			if (rows!=null && cols!=null)
			{
				TableModel tableModel=tblResult.getModel();
				StringBuffer buffer=new StringBuffer();
				for (int i=0; i<rows.length; i++)
				{
					int row=rows[i];
					for (int j=0; j<cols.length; j++)
					{
						int viewColumn=cols[j];
						int modelColumn=tblResult.convertColumnIndexToModel(viewColumn);
						Object cellValue=tableModel.getValueAt(row, modelColumn);
						TableCellRenderer cellRenderer=tblResult.getCellRenderer(row, viewColumn);
						Component component=cellRenderer.getTableCellRendererComponent(tblResult, cellValue, false, false, row, modelColumn);
						if (component instanceof JLabel) cellValue=((JLabel)component).getText();
						if (i+j>0) buffer.append(", ");
						buffer.append(cellValue);
					}
				}
				Clipboard clipboard=Toolkit.getDefaultToolkit().getSystemClipboard();
				clipboard.setContents(new StringSelection(buffer.toString()), null);
			}
		}
	}

	private class CopyColumnNameAction extends AbstractAction
	{
		private String name;

		public CopyColumnNameAction(String name)
		{
			super("Copy Name to Clipboard", IconManager.getIcon(QueryPanel.class.getClassLoader(), IconNames.COPY));
			this.name=name;
		}

		public void actionPerformed(ActionEvent e)
		{
			Clipboard clipboard=Toolkit.getDefaultToolkit().getSystemClipboard();
			clipboard.setContents(new StringSelection(name), null);
		}
	}

	private class ColumnStatisticsAction extends AbstractAction
	{
		private int column;

		public ColumnStatisticsAction(int column)
		{
			super("Statistics", IconManager.getIcon(ResultSetPanel.class.getClassLoader(), IconNames.STATISTIC));
			this.column=column;
		}

		public void actionPerformed(ActionEvent e)
		{
			try
			{
				TableModel model=tblResult.getModel();
				double sum=0;
				double product=1;
				double minimum=Double.NaN;
				double maximum=Double.NaN;
				int rowCount=model.getRowCount();
				List values=new ArrayList();
				for (int row=0; row<rowCount; row++)
				{
					Object object=model.getValueAt(row, column);
					if (object!=null)
					{
						double doubleValue;
						if (object instanceof Number)
							doubleValue=((Number)object).doubleValue();
						else
							doubleValue=Double.parseDouble(object.toString());
						values.add(new Double(doubleValue));
						sum+=doubleValue;
						if (doubleValue<=0)
							product=Double.NaN;
						else
							product*=doubleValue;
						if (Double.isNaN(minimum) || doubleValue<minimum) minimum=doubleValue;
						if (Double.isNaN(maximum) || doubleValue>maximum) maximum=doubleValue;
					}
				}
				double arithmetricMean=Double.NaN;
				double geometricMean=Double.NaN;
				double standardDeviation=Double.NaN;
				double averageDeviation=Double.NaN;
				if (rowCount>0)
				{
					arithmetricMean=sum/rowCount;
					if (!Double.isNaN(product)) geometricMean=Math.pow(product, (double)1/rowCount);
					standardDeviation=0.0;
					averageDeviation=0.0;
					for (Iterator it=values.iterator(); it.hasNext();)
					{
						double value=((Double)it.next()).doubleValue()-arithmetricMean;
						standardDeviation+=value*value;
						averageDeviation+=Math.abs(value);
					}
					standardDeviation=Math.sqrt(standardDeviation/(rowCount-1));
					averageDeviation=averageDeviation/rowCount;
				}

				StatisticsDialogStub dialog=new StatisticsDialogStub("Column Statistics");
				dialog.setNumber(rowCount);
				dialog.setArithmetricMean(arithmetricMean);
				dialog.setGeometricMean(geometricMean);
				dialog.setSum(sum);
				dialog.setMinimum(minimum);
				dialog.setMaximum(maximum);
				dialog.setStandardDeviation(standardDeviation);
				dialog.setAverageDeviation(averageDeviation);
				Utils.wrapDialog(project, dialog, false, true);
			}
			catch (NumberFormatException e1)
			{
				JOptionPane.showMessageDialog(ResultSetPanel.this, "Column contained values not convertable into a number.", "Sum", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	private class CellStatisticsAction extends AbstractAction
	{
		private int[] rows;
		private int[] cols;

		public CellStatisticsAction(int[] rows, int[] cols)
		{
			super("Statistics", IconManager.getIcon(ResultSetPanel.class.getClassLoader(), IconNames.STATISTIC));
			this.rows=rows;
			this.cols=cols;
			setEnabled(rows!=null && rows.length>0 && cols!=null && cols.length>0);
		}

		public void actionPerformed(ActionEvent e)
		{
			if (rows!=null && cols!=null)
			{
				try
				{
					double sum=0;
					double product=1;
					int number=0;
					double minimum=Double.NaN;
					double maximum=Double.NaN;
					List values=new ArrayList();
					for (int i=0; i<rows.length; i++)
					{
						int row=rows[i];
						for (int j=0; j<cols.length; j++)
						{
							int col=cols[j];
							Object object=tblResult.getModel().getValueAt(row, col);
							if (object!=null)
							{
								number++;
								double value;
								if (object instanceof Number)
									value=((Number)object).doubleValue();
								else
									value=Double.parseDouble(object.toString());
								values.add(new Double(value));
								sum+=value;
								if (value<=0)
									product=Double.NaN;
								else
									product*=value;
								if (Double.isNaN(minimum) || value<minimum) minimum=value;
								if (Double.isNaN(maximum) || value>maximum) maximum=value;
							}
						}
					}
					double arithmetricMean=Double.NaN;
					double geometricMean=Double.NaN;
					double standardDeviation=Double.NaN;
					double averageDeviation=Double.NaN;
					if (number>0)
					{
						arithmetricMean=sum/number;
						if (!Double.isNaN(product)) geometricMean=Math.pow(product, (double)1/number);
						standardDeviation=0.0;
						averageDeviation=0.0;
						for (Iterator it=values.iterator(); it.hasNext();)
						{
							double value=((Double)it.next()).doubleValue()-arithmetricMean;
							standardDeviation+=value*value;
							averageDeviation+=Math.abs(value);
						}
						standardDeviation=Math.sqrt(standardDeviation/(number-1));
						averageDeviation=averageDeviation/number;
					}

					StatisticsDialogStub dialog=new StatisticsDialogStub("Cell Statistics");
					dialog.setNumber(number);
					dialog.setArithmetricMean(arithmetricMean);
					dialog.setGeometricMean(geometricMean);
					dialog.setSum(sum);
					dialog.setMinimum(minimum);
					dialog.setMaximum(maximum);
					dialog.setStandardDeviation(standardDeviation);
					dialog.setAverageDeviation(averageDeviation);
					Utils.wrapDialog(project, dialog, false, true);
				}
				catch (NumberFormatException e1)
				{
					JOptionPane.showMessageDialog(ResultSetPanel.this, "Selection contained values not convertable into a number.", "Sum", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}

	private class SetFormatAction extends AbstractAction
	{
		private int column;
		private String format;

		public SetFormatAction(int column, String format)
		{
			super(format);
			this.column=column;
			this.format=format;
		}

		public void actionPerformed(ActionEvent e)
		{
			if (tblResult.getModel() instanceof ResultSetTableModel)
			{
				ResultSetTableModel model=(ResultSetTableModel)tblResult.getModel();
				model.setColumnFormat(column, format);
				tblResult.repaint();
			}
		}
	}

	private class SetFontAction extends AbstractAction
	{
		private int column;
		private String currentFont;

		public SetFontAction(int column, String currentFont)
		{
			super("Font", IconManager.getIcon(ResultSetPanel.class.getClassLoader(), "icons/font.gif"));
			this.column=column;
			this.currentFont=currentFont;
		}

		public void actionPerformed(ActionEvent e)
		{
			if (tblResult.getModel() instanceof ResultSetTableModel)
			{
				ResultSetTableModel model=(ResultSetTableModel)tblResult.getModel();
				FontSelectionDialogStub dialog=new FontSelectionDialogStub(currentFont);
				Utils.wrapDialog(project, dialog, true, true);
				if (dialog.getReturnValue())
				{
					if (dialog.getFont()!=null)
						model.setColumnFont(column, new Font(dialog.getFont(), Font.PLAIN, tblResult.getFont().getSize()));
					else
						model.setColumnFont(column, null);
					tblResult.repaint();
				}
			}
		}
	}

	private class RefreshAction extends AbstractAction
	{
		public RefreshAction()
		{
			super("Refresh Data", IconManager.getIcon(ResultSetPanel.class.getClassLoader(), "icons/sync.png"));
		}

		public void actionPerformed(ActionEvent e)
		{
			try
			{
				SQLStatement statement=((ResultSetTableModel)tblResult.getModel()).getStatement();
				queryPanel.executeStatement(statement, ResultSetPanel.this);
			}
			catch (Exception e1)
			{
				e1.printStackTrace();
			}
		}
	}

	private class AutoRefreshAction extends AbstractAction
	{
		public AutoRefreshAction()
		{
			super("Auto-Refresh Data", IconManager.getIcon(ResultSetPanel.class.getClassLoader(), "icons/sync.png"));
			if (autoRefreshThread!=null && autoRefreshThread.isAlive())
			{
				putValue(Action.NAME, "Stop Auto-Refresh");
			}
		}

		public void actionPerformed(ActionEvent e)
		{
			if (autoRefreshThread==null || !autoRefreshThread.isAlive())
			{
				String value=(String)JOptionPane.showInputDialog(ResultSetPanel.this,
						"Time between 2 executions (s):", "Interval", JOptionPane.QUESTION_MESSAGE, null, null, "1");
				if (value!=null)
				{
					try
					{
						final int interval=Integer.parseInt(value);
						if (interval<=0) throw new NumberFormatException();
						autoRefreshThread=new AutoRefreshThread(interval*1000);
						autoRefreshThread.start();
					}
					catch (NumberFormatException e1)
					{
						JOptionPane.showMessageDialog(ResultSetPanel.this, "Invalid interval.", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
			}
			else if (autoRefreshThread!=null)
			{
				autoRefreshThread.setStopped(true);
			}
		}
	}

	public DynamicTable getResultTable()
	{
		return tblResult;
	}

	public void setTableModel(ResultSetTableModel model)
	{
		tblResult.setModel(model);
		ResultTableConfiguration tableConfiguration=model.getTableConfiguration();
		CommonConfiguration configuration=CommonConfiguration.getInstance(project);
		tblResult.initializeColumns(tableConfiguration);
		if (!tableConfiguration.isSupportWidths())
			tblResult.sizeColumnsToFit(configuration.isResizeColumnsToHeader(), configuration.isResizeColumnsToContent());
		tableConfiguration.setInitalized(true);
	}

	private class AutoRefreshThread extends Thread
	{
		private boolean stopped;
		private final int interval;
		private boolean interuptable;

		public AutoRefreshThread(int interval)
		{
			this.interval=interval;
		}

		public void setStopped(boolean stopped)
		{
			this.stopped=stopped;
			if (interuptable) interrupt();
		}

		public void run()
		{
			try
			{
				while (!stopped)
				{
					interuptable=false;
					Container topLevelAncestor=ResultSetPanel.this.getTopLevelAncestor();
					if (topLevelAncestor!=null && topLevelAncestor.isVisible())
					{
						SQLStatement statement=((ResultSetTableModel)tblResult.getModel()).getStatement();
						queryPanel.executeStatement(statement, ResultSetPanel.this);
						if (queryPanel.getErrors()) break;
					}
					interuptable=true;
					try
					{
						sleep(interval);
					}
					catch (InterruptedException e1)
					{
						break;
					}
				}
			}
			catch (Throwable e1)
			{
				e1.printStackTrace();
			}
		}
	}

	public SQLStatement getStatement()
	{
		if (tblResult.getModel() instanceof ResultSetTableModel)
		{
			return ((ResultSetTableModel)tblResult.getModel()).getStatement();
		}
		return null;
	}
}
