/*
 * Copyright (C) 2002-2003 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin;

import java.awt.Font;
import java.awt.Color;
import java.sql.*;
import java.util.*;
import java.util.Date;

import com.intellij.openapi.project.Project;
import com.kiwisoft.db.DataConverter;
import com.kiwisoft.db.Database;
import com.kiwisoft.db.DatabaseUtils;
import com.kiwisoft.db.ColumnData;
import com.kiwisoft.db.driver.DatabaseDriverManager;
import com.kiwisoft.db.sql.SQLStatement;
import com.kiwisoft.utils.DateUtils;
import com.kiwisoft.utils.ObjectUtils;
import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.SetMap;
import com.kiwisoft.utils.format.ObjectFormat;
import com.kiwisoft.utils.gui.ObjectStyle;
import com.kiwisoft.utils.gui.table.DefaultSortableTableRow;
import com.kiwisoft.utils.gui.table.SortableTableModel;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.6 $, $Date: 2005/04/06 20:14:49 $
 */
public class ResultSetTableModel extends SortableTableModel
{
	private Vector columnInfos;
	private QueryPanel queryPanel;
	private Project project;
	private Database database;
	private SQLStatement statement;
	private boolean editable;
	private Set primaryKeyColumns;

	private boolean incomplete;
	private ClassLoader projectClassLoader;
	private ResultTableConfiguration tableConfiguration;

	private final static ObjectStyle MODIFIED_STYLE=new ObjectStyle(null, new Color(215, 215, 255));

	public ResultSetTableModel(QueryPanel queryPanel, Project project, Database database, SQLStatement statement, ResultSet resultSet,
			ClassLoader classLoader) throws Exception
	{
		super();
		this.queryPanel=queryPanel;
		this.project=project;
		this.projectClassLoader=classLoader;
		this.database=database;
		this.statement=statement;
		primaryKeyColumns=new HashSet();
		createData(resultSet);
	}

	public SQLStatement getStatement()
	{
		return statement;
	}

	private void createData(ResultSet resultSet) throws Exception
	{
		CommonConfiguration
				commonConfiguration=CommonConfiguration.getInstance(project);
		ResultSetMetaData metaData=resultSet.getMetaData();

		columnInfos=new Vector();
		editable=statement.isSimpleSelect();
		tableConfiguration=database.getResultTableConfiguration(statement);

		Map tableReferences=new HashMap();

		for (int i=0; i<metaData.getColumnCount(); i++)
		{
			int column=i+1;
			String columnName=metaData.getColumnName(column);
			ColumnInfo columnInfo=new ColumnInfo(columnName);
            columnInfos.add(columnInfo);

			if (commonConfiguration.isShowPrimaryKeyColumns())
			{
				int identifierCase=database.getIdentifiersCase(project);
				try
				{
					String catalogName=getCatalogName(metaData, column);
					String schemaName=getSchemaName(metaData, column);
					String tableName=getTableName(metaData, column);
					if (editable)
					{
						if (StringUtils.isEmpty(tableName))
							tableName=StringUtils.applyCharacterCase(statement.getTableName(), identifierCase);
						if (StringUtils.isEmpty(schemaName))
							schemaName=database.getDefaultSchema(project).getSchemaName();
						if (StringUtils.isEmpty(catalogName))
							catalogName=database.getCatalog(project);
					}
					columnInfo.setSchemaName(schemaName);

					if (columnName!=null)
					{
						String referenceKey=catalogName+"."+schemaName+"."+tableName;
						TableReferenceData referenceData=(TableReferenceData)tableReferences.get(referenceKey);
						if (referenceData==null) tableReferences.put(referenceKey, referenceData=new TableReferenceData());
						String actualColumnName=StringUtils.applyCharacterCase(columnName, identifierCase);
						if (referenceData.primaryKeys==null)
							referenceData.primaryKeys=database.getPrimaryKeys(project, catalogName, schemaName, tableName);
						columnInfo.setPrimaryKey(referenceData.primaryKeys.contains(actualColumnName));
						if (referenceData.importedKeys==null)
							referenceData.importedKeys=database.getImportedKeys(project, catalogName, schemaName, tableName);
						columnInfo.setForeignKey((ColumnData)referenceData.importedKeys.get(actualColumnName));
						if (columnInfo.isPrimaryKey())
						{
							if (referenceData.exportedKeys==null)
								referenceData.exportedKeys=database.getExportedKeys(project, catalogName, schemaName, tableName);
							columnInfo.setExportedKeys(referenceData.exportedKeys.get(actualColumnName));
						}
					}
					if (columnInfo.isPrimaryKey()) primaryKeyColumns.add(new Integer(i));
				}
				catch (Throwable e)
				{
					e.printStackTrace();
					System.err.println("Error while getting primary/foreign keys: "+e.getMessage());
				}
			}
			Class columnType=null;
			try
			{
				columnInfo.setJdbcType(metaData.getColumnType(column));
				String columnClassName=metaData.getColumnClassName(column);
				if ("byte[]".equals(columnClassName)) columnType=byte[].class;
				else columnType=Class.forName(columnClassName, true, DatabaseDriverManager.getClassLoader());
				if (columnType!=null) columnType=DataConverter.convertClass(columnType);
			}
			catch (Throwable e)
			{
				System.err.println("Error while getting column class name: "+e.getMessage());
			}
			columnInfo.setType(columnType);
			if (tableConfiguration!=null)
			{
				columnInfo.setFormat(tableConfiguration.getFormat(i));
				columnInfo.setFont(tableConfiguration.getFont(i));
			}
		}

		boolean limitRows=commonConfiguration.isRowLimitEnabled();
		int rowLimit=commonConfiguration.getRowLimit();
		boolean hasNext=resultSet.next();
		while (hasNext && (!limitRows || getRowCount()<rowLimit))
		{
			Vector row=new Vector();
			for (int i=0; i<metaData.getColumnCount(); i++)
			{
				Object object;
				try
				{
					object=resultSet.getObject(i+1);
					object=DataConverter.convertValue(object, projectClassLoader, commonConfiguration.isLoadLargeObjects());
				}
				catch (SQLException e)
				{
					object="<Read Error>";
				}
				row.add(object);

				if (object!=null)
				{
					ColumnInfo columnInfo=getColumnInfo(i);
					columnInfo.setType(ObjectUtils.getBaseClass(columnInfo.getType(), object.getClass()));
				}
			}
			addRow(new TableRow(row));
			hasNext=resultSet.next();
		}
		if (hasNext) incomplete=true;
	}

	private static String getTableName(ResultSetMetaData metaData, int column)
	{
		String tableName=null;
		try
		{
			tableName=metaData.getTableName(column);
		}
		catch (SQLException e)
		{
			System.err.println("getTableName not supported: "+e.getMessage());
		}
		return tableName;
	}

	private static String getSchemaName(ResultSetMetaData metaData, int column)
	{
		String schemaName=null;
		try
		{
			schemaName=metaData.getSchemaName(column);
		}
		catch (SQLException e)
		{
			System.err.println("getSchemaName not supported: "+e.getMessage());
		}
		return schemaName;
	}

	private static String getCatalogName(ResultSetMetaData metaData, int column)
	{
		String catalogName=null;
		try
		{
			catalogName=metaData.getCatalogName(column);
		}
		catch (Exception e)
		{
			System.err.println("getCatalogName not supported: "+e.getMessage());
		}
		return catalogName;
	}

	public boolean isIncomplete()
	{
		return incomplete;
	}

	public int getColumnCount()
	{
		return columnInfos.size();
	}

	public String getColumnName(int col)
	{
		return getColumnInfo(col).getName();
	}

	public void setColumnFormat(int col, String format)
	{
		getColumnInfo(col).setFormat(format);
		tableConfiguration.setFormat(col, format);
	}

	public ColumnInfo getColumnInfo(int col)
	{
		return ((ColumnInfo)columnInfos.get(col));
	}

	public String getCellFormat(int row, int col)
	{
		ColumnInfo info=getColumnInfo(col);
		String format=info.getFormat();
		Class type=info.getType();
		if (type!=null && (format==null || ObjectFormat.DEFAULT.equals(format)))
		{
			String defaultFormat=null;
			CommonConfiguration configuration=CommonConfiguration.getInstance(project);
			if (Date.class.isAssignableFrom(type)) defaultFormat=configuration.getDefaultFormat(Date.class);
			else if (Number.class.isAssignableFrom(type)) defaultFormat=configuration.getDefaultFormat(Number.class);
			else if (String.class.isAssignableFrom(type)) defaultFormat=configuration.getDefaultFormat(String.class);
			else if (Boolean.class.isAssignableFrom(type)) defaultFormat=configuration.getDefaultFormat(Boolean.class);
			if (defaultFormat!=null) return defaultFormat;
		}
		return format;
	}

	public ObjectStyle getCellStyle(int row, int col)
	{
		ObjectStyle style1=getColumnInfo(col).getStyle();
		ObjectStyle style2=super.getCellStyle(row, col);
		if (style1==null) return style2;
		if (style2==null) return style1;
		return style1.combine(style2);
	}

	public void setColumnFont(int col, Font font)
	{
		getColumnInfo(col).setFont(font);
		tableConfiguration.setFont(col, font);
	}

	public Class getCellClass(int row, int col)
	{
		Class type=getColumnInfo(col).getType();
		if (type==null) type=Object.class;
		return type;
	}

	public boolean isPrimaryKey(int col)
	{
		return getColumnInfo(col).isPrimaryKey();
	}

	public void setPrimaryKey(int col, boolean value)
	{
		getColumnInfo(col).setPrimaryKey(value);
		if (value)
			primaryKeyColumns.add(new Integer(col));
		else
			primaryKeyColumns.remove(new Integer(col));
		fireTableDataChanged();
	}

	private class TableRow extends DefaultSortableTableRow
	{
		private boolean modified;

		public TableRow(Vector data)
		{
			super(data);
		}

		public ObjectStyle getCellStyle(int col)
		{
			if (modified) return MODIFIED_STYLE;
			return super.getCellStyle(col);
		}

		public int setValue(Object value, int col)
		{
			if ("".equals(value)) value=null;
			if (value!=null)
			{
				switch (getColumnInfo(col).getJdbcType())
				{
					case Types.DATE:
						if (value instanceof String)
						{
							Date date=DateUtils.parseDate((String)value);
							if (date==null) return NO_UPDATE;
							else value=date;
						}
						if (value instanceof Date && !(value instanceof java.sql.Date))
							value=new java.sql.Date(((Date)value).getTime());
						break;
					case Types.TIMESTAMP:
						if (value instanceof String)
						{
							Date date=DateUtils.parseDate((String)value);
							if (date==null) return NO_UPDATE;
							else value=date;
						}
						if (value instanceof Date && !(value instanceof java.sql.Timestamp)) 
							value=new java.sql.Timestamp(((Date)value).getTime());
				}
			}
			Vector parameters=new Vector();
			StringBuffer updateStatement=new StringBuffer("update ");
			updateStatement.append(statement.getTableName());
			updateStatement.append(" set ");
			updateStatement.append((getColumnInfo(col)).getName());
			updateStatement.append("=?");
			parameters.add(value);
			updateStatement.append(" where ");
			Iterator it=primaryKeyColumns.iterator();
			while (it.hasNext())
			{
				int column=((Integer)it.next()).intValue();
				updateStatement.append(getColumnInfo(column).getName());
				updateStatement.append("=?");
				Object displayValue=getDisplayValue(column);
				parameters.add(displayValue);
				if (it.hasNext()) updateStatement.append(" and ");
			}
			Connection connection=DatabaseUtils.connect(project, database);
			try
			{
				PreparedStatement stmt=connection.prepareStatement(updateStatement.toString());
				for (int i=0; i<parameters.size(); i++)
				{
					Object parameter=parameters.get(i);
					if (parameter instanceof Number) parameter=parameter.toString();
					stmt.setObject(i+1, parameter);
				}
				queryPanel.showMessage(bindParameters(updateStatement.toString(), parameters), QueryPanel.INFO, false, false);
				int updateCount=stmt.executeUpdate();
				queryPanel.showMessage(updateCount+" row(s) updated.", QueryPanel.DEFAULT, true, updateCount!=1);
				if (updateCount>0)
				{
					((Vector)getUserObject()).set(col, value);
					modified=true;
					return ROW_UPDATE;
				}
				return NO_UPDATE;
			}
			catch (SQLException e)
			{
				e.printStackTrace();
				queryPanel.handleThrowable(e);
				return ROW_UPDATE;
			}
		}

		public boolean isEditable(int column)
		{
			return editable && !primaryKeyColumns.isEmpty() &&
					getColumnInfo(column).isEditable();
		}
	}

	private static String bindParameters(String statement, List paramenters)
	{
		StringBuffer buffer=new StringBuffer();
		Iterator itParameters=paramenters.iterator();
		for (int i=0;i<statement.length();i++)
		{
			char ch=statement.charAt(i);
			if (ch=='?') buffer.append("?/*").append(itParameters.next()).append("*/");
			else buffer.append(ch);
		}
		return buffer.toString();
	}

	public ResultTableConfiguration getTableConfiguration()
	{
		return tableConfiguration;
	}

	private static class TableReferenceData
	{
		public Set primaryKeys;
		public Map importedKeys;
		public SetMap exportedKeys;
	}
}
